#!/bin/bash
#
# Copyright 2012 Doerzbach Engineering GmbH 
#
# http://www.doerzbach.com
# 
# Written by Armin Doerzbach 
# 

cwd=`pwd`
cd `dirname $0`
bindir=`pwd`
basedir=`dirname $bindir`
cd $cwd

connectssh() {
  ssh -o "BatchMode yes" -o "StrictHostKeyChecking no" -i $basedir/etc/ssh_keys/$srcuser/$svcsname $dstuser@$dsthost $* 
}

if [ -z "$1" ] ; then
  echo "Usage: $(basename $0) [-i|-r responsefile]"
  exit 1
fi

filename=
if [ $1 = -i ] ; then
  echo 'ZFSREPL: Copyright 2012 Doerzbach Engineering GmbH'
  echo '         Written by Armin Doerbach'
  echo '         http://www.doerzbach.com'
  echo ''
  echo 'THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND'
  echo 'ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,'
  echo 'THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR '
  echo 'A PARTICULAR PURPOSE ARE DISCLAIMED. '
  echo 'IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,'
  echo 'SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,'
  echo 'PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;'
  echo 'OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, '
  echo 'WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)'
  echo 'ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED '
  echo 'OF THE POSSIBILITY OF SUCH DAMAGE.'
  echo 'Please read the statement above carefully. Do you agree with it (YES/no)?'
  read an
  if [ "$an" != "YES" ] ; then
    exit 1
  fi
  while [ -z "$filename" ] || ! touch $filename ; do
    echo -n "Enter the filename of the responsefile you want to create:"
    read filename
  done
else
  if [ $1 = -r -a -r "$2" ] ; then
    filename=$2
  else
    echo "Can not read $2"
    exit 2
  fi
fi
if [ "$filename" = "${filename#/}" ] ; then
  filename=$cwd/$filename 
fi
. $filename
if [ $1 = -i ] ; then
  cp $filename $filename.`date +%Y%m%d%H%M%S`
  cp /dev/null $filename
fi
# get local user
if [ -z "$srcuser" ] ; then
  srcuser=`id | sed 's/[^(]*(\([^)]*\).*/\1/g'`
  echo srcuser=$srcuser >> $filename
else
  thisuser=`id | sed 's/[^(]*(\([^)]*\).*/\1/g'`
  if [ "$thisuser" != "$srcuser" ] ; then
    echo the user in the file $filename is not the same than the user logged in now
    exit 1
  fi
  echo srcuser=$srcuser >> $filename
fi
# get remote host
an=
[ $1 = -r -a -n "$srchost" ] && an=bla
while [ -z "$an" ] ; do
  echo -n "Enter the source system name (logical alias name for a cluster) which will send the snapshots [default:$srchost]:"
  read an
  if [ -n "$an" ] ; then
    srchost=$an
  else
    an=$srchost
  fi
  if ! getent hosts $srchost >/dev/null ; then
    an=
    srchost=
  else 
    an=$srchost
    echo srchost=$srchost >> $filename
  fi
done  
# get remote host
an=
[ $1 = -r -a -n "$dsthost" ] && an=bla
while [ -z "$an" ] ; do
  echo -n "Enter the destination system name (logical alias name for a cluster) which will receive the snapshots [default:$dsthost]:"
  read an
  if [ -n "$an" ] ; then
    dsthost=$an
  else
    an=$dsthost
  fi
  if ! getent hosts $dsthost >/dev/null ; then
    an=
    dsthost=
  else 
    an=$dsthost
    echo dsthost=$dsthost >> $filename
  fi
done  
# get remote user
an=
[ $1 = -r -a -n "$dstuser" ] && an=bla
while [ -z "$an" ] ; do
  echo -n "Enter the username on the destination system which will receive the snapshots [default:$dstuser]:"
  read an
  if [ -n "$an" ] ; then
    dstuser=$an
  else
    an=dstuser
  fi
  echo "The user id on the destination system will be checked now."
  echo "Please enter the password of the destination user if asked..."
  if ! ssh -i $basedir/etc/ssh_keys/$srcuser/$svcsname $dstuser@$dsthost "exit 0" >/dev/null ; then
    an=
    dstuser=
  else 
    an=$dstuser
    echo dstuser=$dstuser >> $filename
  fi
done  
# get number of filesystems to sync
an=
[ $1 = -r -a -n "$number_of_filesytems" ] && an=bla
while [ -z "$an" ] ; do
  echo -n "How many filesystems do you want to replicate with this service [default:$number_of_filesytems]:"
  read an
  if [ -n "$an" ] ; then
    number_of_filesytems=$an
  else
    an=$number_of_filesytems
  fi
  if [ -n "${number_of_filesytems##[0-9]*}" ] ; then
    an=
    number_of_filesytems=
  else 
    an=$number_of_filesytems
    echo number_of_filesytems=$number_of_filesytems >> $filename
  fi
done  
# loop of fileystems
index=0
while [ $index -lt $number_of_filesytems ] ; do
  let index++
  an=
  [ $1 = -r -a -n "${source_zfs[$index]}" ] && an=bla
  while [ -z "$an" ] ; do
    echo -n "Enter source zfs filesystem[$index] you want to replicate [default:${source_zfs[$index]}]:"
    read an
    if [ -n "$an" ] ; then
      source_zfs[$index]=$an
    else
      an=${source_zfs[$index]}
    fi
    if ! /usr/sbin/zfs list ${source_zfs[$index]} >/dev/null 2>&1 ; then
      an=
      source_zfs[$index]=
    else 
      an=${source_zfs[$index]}
      echo source_zfs[$index]=${source_zfs[$index]} >> $filename
    fi
  done  
  an=
  [ $1 = -r -a -n "${dest_zfs[$index]}" ] && an=bla
  while [ -z "$an" ] ; do
    echo -n "Enter destination zfs filesystem[$index] [default:${dest_zfs[$index]}]:"
    read an
    if [ -n "$an" ] ; then
      dest_zfs[$index]=$an
    else
      an=${dest_zfs[$index]}
    fi
    echo "Going to check if the parent filesystem `dirname ${dest_zfs[$index]}` of the destination filesystem ${dest_zfs[$index]} exists."
    echo "Enter the passwort of the destination user if asked..."
    if ! ssh -i $basedir/etc/ssh_keys/$srcuser/$svcsname $dstuser@$dsthost "/usr/sbin/zfs list `dirname ${dest_zfs[$index]}`" >/dev/null 2>&1 ; then
      an=
      dest_zfs[$index]=
    else 
      an=${dest_zfs[$index]}
      echo dest_zfs[$index]=${dest_zfs[$index]} >> $filename
    fi
  done  
done

# Get interval
an=
[ $1 = -r -a -n "$interval" ] && an=bla
while [ -z "$an" ] ; do
  echo -n "Enter the replication interval in seconds [default:$interval]:"
  read an
  if [ -n "$an" ] ; then
    interval=$an
  else
    an=$interval
  fi
  if echo $interval | grep '^[0-9]*$' >/dev/null ; then
    an=$interval
    echo interval=$interval >> $filename
  else 
    an=
    interval=
  fi
done  
# Get the number of retries to make before failing 
an=
[ -z "$retries" ] && retries=1
[ $1 = -r -a -n "$retries" ] && an=bla
while [ -z "$an" ] ; do
  echo -n "Enter the number of retries before failing the script or service [default:$retries]:"
  read an
  if [ -n "$an" ] ; then
    retries=$an
  else
    an=$retries
  fi
  if echo $retries | grep '^[0-9]*$' >/dev/null ; then
    an=$retries
    echo retries=$retries >> $filename
  else 
    an=
    retries=
  fi
done  
# Get the number of snaps to keep
an=
[ -z "$keepsnaps" ] && keepsnaps=1
[ $1 = -r -a -n "$keepsnaps" ] && an=bla
while [ -z "$an" ] ; do
  echo -n "Enter the number of snapshots to keep before deleting them [default:$keepsnaps]:"
  read an
  if [ -n "$an" ] ; then
    keepsnaps=$an
  else
    an=$keepsnaps
  fi
  if echo $keepsnaps | grep '^[0-9]*$' >/dev/null ; then
    an=$keepsnaps
    echo keepsnaps=$keepsnaps >> $filename
  else 
    an=
    keepsnaps=
  fi
done  
# Get beforescript
an=
[ $1 = -r -a -n "$beforescript" ] && an=bla
while [ -z "$an" ] ; do
  echo -n "Enter the full path to the script to run before the snapshot should be taken (none if no script should be taken) [default:$beforescript]:"
  read an
  if [ -n "$an" ] ; then
    beforescript=$an
  else
    an=$beforescript
  fi
  if [ "$beforescript" != "" ] ; then
    if [ "$beforescript" = "none" -o -x "$beforescript" ] ; then
      an=$beforescript
      echo beforescript=$beforescript >> $filename
    else 
      an=
      beforescript=
    fi
  fi
done  
# Get afterscript
an=
[ $1 = -r -a -n "$afterscript" ] && an=bla
while [ -z "$an" ] ; do
  echo -n "Enter the full path to the script to run after the snapshot should be taken (none if no script should be taken) [default:$afterscript]:"
  read an
  if [ -n "$an" ] ; then
    afterscript=$an
  else
    an=$afterscript
  fi
  if [ "$afterscript" != "" ] ; then
    if [ "$afterscript" = "none" -o -x "$afterscript" ] ; then
      an=$afterscript
      echo afterscript=$afterscript >> $filename
    else 
      an=
      afterscript=
    fi
  fi
done  
# Get servicename
an=
[ $1 = -r -a -n "$svcsname" ] && an=bla
while [ -z "$an" ] ; do
  echo -n "Enter the service name which should be created [default:$svcsname]:"
  read an
  if [ -n "$an" ] ; then
    svcsname=$an
  else
    an=$svcsname
  fi
  if echo $servicename | grep '^[a-zA-Z]*$' >/dev/null ; then
    an=$svcsname
    echo svcsname=$svcsname >> $filename
  else 
    an=
    svcsname=
  fi
done  
echo "Do you want to make changes now (Y/[n])?"
read an
if [ "$an" != "Y" ] ;then 
  exit 1
fi

rootscriptname=doit_as_root.$svcsname.`uname -n`.bash
remoterootscriptname=doit_as_root.$svcsname.$dstuser.bash
# Genreate root scripts
echo "#!/bin/bash" > $HOME/$rootscriptname
echo "set -e" >> $HOME/$rootscriptname
echo "PATH=/usr/sbin:/usr/bin" >> $HOME/$rootscriptname
echo "#!/bin/bash" > $HOME/$remoterootscriptname
echo "set -e" >> $HOME/$remoterootscriptname
echo "PATH=/usr/sbin:/usr/bin" >> $HOME/$remoterootscriptname
echo "echo Creating directory for the public and private keys $basedir/etc/ssh_keys/$srcuser" >> $HOME/$rootscriptname
echo "[ -d $basedir/etc/ssh_keys/$srcuser ] || mkdir $basedir/etc/ssh_keys/$srcuser" >> $HOME/$rootscriptname
echo chown -R $srcuser $basedir/etc/ssh_keys/$srcuser >> $HOME/$rootscriptname
echo chmod 700 $basedir/etc/ssh_keys/$srcuser >> $HOME/$rootscriptname
echo chmod 700 $basedir/etc/ssh_keys/$srcuser >> $HOME/$rootscriptname
echo "echo Creating directory for the public and private keys $basedir/etc/ssh_keys/$dstuser" >> $HOME/$remoterootscriptname
echo "[ -d $basedir/etc/ssh_keys/$dstuser ] || mkdir $basedir/etc/ssh_keys/$dstuser" >> $HOME/$remoterootscriptname
echo chown -R $dstuser $basedir/etc/ssh_keys/$dstuser >> $HOME/$remoterootscriptname
echo chmod 700 $basedir/etc/ssh_keys/$dstuser >> $HOME/$remoterootscriptname
echo chmod 700 $basedir/etc/ssh_keys/$dstuser >> $HOME/$remoterootscriptname
# generate commands zfs allow on local and remote filesystems
index=0
echo "echo Running zfs allow -u $srcuser create,destroy,mount,send,snapshot,receive,compression,recordsize on ..." >> $HOME/$rootscriptname
echo "echo Running zfs allow -u $dstuser create,destroy,mount,send,snapshot,receive,compression,recordsize  and chown $dstuser on ..." >> $HOME/$remoterootscriptname
while [ $index -lt $number_of_filesytems ] ; do
  let index++
  echo "echo `dirname ${source_zfs[$index]}`" >> $HOME/$rootscriptname
  echo zfs allow -u $srcuser create,destroy,mount,send,snapshot,receive,compression,recordsize `dirname ${source_zfs[$index]}` >> $HOME/$rootscriptname
  echo "echo `dirname ${dest_zfs[$index]}`" >> $HOME/$remoterootscriptname
  echo zfs allow -u $dstuser create,destroy,mount,send,snapshot,receive,compression,recordsize `dirname ${dest_zfs[$index]}` >> $HOME/$remoterootscriptname
  echo mp=\`/usr/sbin/zfs list -o mountpoint -H $(dirname ${dest_zfs[$index]})\` >> $HOME/$remoterootscriptname
  echo '[ "$mp" != "${mp#/}" ] &&' chown $dstuser \$mp >> $HOME/$remoterootscriptname
done
# commands to generate service only if interval>0
if [ $interval -ne 0 ] ; then
  service=application/zfsrepl
  if ! /usr/sbin/svccfg -s $service list >/dev/null 2>&1 ; then
    cat <<EOF >>$HOME/$rootscriptname
svccfg add $service
svccfg -s $service addpg multi_user dependency
svccfg -s $service setprop multi_user/entities=fmri: "svc:/milestone/multi-user"
svccfg -s $service setprop multi_user/grouping=astring: "require_all"
svccfg -s $service setprop multi_user/restart_on=astring: "none"
svccfg -s $service setprop multi_user/type=astring: "service"
svccfg -s $service addpg stop method 
svccfg -s $service setprop stop/exec=astring: ":kill"
svccfg -s $service setprop stop/project=astring: ":default"
svccfg -s $service setprop stop/resource_pool=astring: ":default"
svccfg -s $service setprop stop/timeout_seconds=count: 60
svccfg -s $service setprop stop/type=astring: "method"
svccfg -s $service setprop stop/working_directory=astring: ":default"
EOF
  fi
  if svcs $service:$svcsname >/dev/null 2>&1 ; then
     echo svcadm disable -s $service:$svcsname >> $HOME/$rootscriptname
     echo svccfg delete $service:$svcsname >> $HOME/$rootscriptname
  fi   
  cat <<EOF >>$HOME/$rootscriptname
svccfg -s $service add $svcsname
svccfg -s $service:$svcsname addpg start method
svccfg -s $service:$svcsname setprop start/group=astring: ":default"
svccfg -s $service:$svcsname setprop start/limit_privileges=astring: ":default"
svccfg -s $service:$svcsname setprop start/privileges=astring: ":default"
svccfg -s $service:$svcsname setprop start/project=astring: ":default"
svccfg -s $service:$svcsname setprop start/resource_pool=astring: ":default"
svccfg -s $service:$svcsname setprop start/supp_groups=astring: ":default"
svccfg -s $service:$svcsname setprop start/timeout_seconds=count: 60
svccfg -s $service:$svcsname setprop start/type=astring: "method"
svccfg -s $service:$svcsname setprop start/use_profile=boolean: "false"
svccfg -s $service:$svcsname setprop start/user=astring: "$srcuser"
svccfg -s $service:$svcsname setprop start/working_directory=astring: ":default"
svccfg -s $service:$svcsname setprop start/environment=astring: \"LOGLEVEL=WARNING\"
svccfg -s $service:$svcsname setprop start/exec=astring: "\"$bindir/zfs_replicate_service.bash start %i"\"
svcadm disable $service:$svcsname 2>/dev/null 1>&2 || echo 
EOF
fi
echo cp $filename $basedir/etc/svcsdefinition/$svcsname >> $HOME/$rootscriptname
echo cp /tmp/$svcsname.remote $basedir/etc/svcsdefinition/$svcsname.remote >> $HOME/$remoterootscriptname
echo touch /tmp/$rootscriptname.done >> $HOME/$rootscriptname
echo chown $srcuser /tmp/$rootscriptname.done >> $HOME/$rootscriptname
echo touch /tmp/$remoterootscriptname.done >> $HOME/$remoterootscriptname
echo chown $dstuser /tmp/$remoterootscriptname.done >> $HOME/$remoterootscriptname
echo rm /tmp/$remoterootscriptname >> $HOME/$remoterootscriptname
chmod 755 $HOME/$remoterootscriptname $HOME/$rootscriptname
# Let the local user run the rootscript $rootscriptname

echo -n "Please run the script $HOME/$rootscriptname as root on the source system $srchost now from another terminal."
while [ ! -f /tmp/$rootscriptname.done ] ; do
  sleep 5
  echo -n .
done
echo "Thank you!!!"
sleep 10
rm /tmp/$rootscriptname.done

# generate ssh keyfile
if [ ! -f $basedir/etc/ssh_keys/$srcuser/$svcsname ] ; then
    ssh-keygen -t rsa -N "" -C "zfs_replication:$svcsname@$srchost" -f $basedir/etc/ssh_keys/$srcuser/$svcsname 
fi
# install public key for remote user

if ! connectssh "exit 0" 2>/dev/null 1>&2 ; then
  echo "Installing public key on remote system now..."
  echo "Please enter the password if asked."
  cat $basedir/etc/ssh_keys/$srcuser/$svcsname.pub | ssh $dstuser@$dsthost "mkdir -p .ssh; cat >>.ssh/authorized_keys"
fi
# copy zfs allow script as root on remote system
scp -i $basedir/etc/ssh_keys/$srcuser/$svcsname $HOME/$remoterootscriptname $dstuser@$dsthost:/tmp
scp -i $basedir/etc/ssh_keys/$srcuser/$svcsname $filename $dstuser@$dsthost:/tmp/$svcsname.remote

# let the user run to remote script
echo -n "Please run the script /tmp/$remoterootscriptname as root on the remote host $dsthost now."
while ! connectssh "ls /tmp/$remoterootscriptname.done" >/dev/null 2>&1 ; do 
  sleep 5
  echo -n .
done
echo "Thank you!!!"
sleep 10
connectssh "rm /tmp/$remoterootscriptname.done"

if ! connectssh "ls $basedir/etc/ssh_keys/$dstuser/$svcsname 2>/dev/null 1>&2" ; then
  connectssh "ssh-keygen -t rsa -N '' -C 'zfs_replication:$svcsname@$dsthost' -f $basedir/etc/ssh_keys/$dstuser/$svcsname"
fi
# check allow
echo "check installation now..."
$bindir/zfs_replicate_service.bash check $svcsname
if [ $? -eq 0 ] ; then
  echo "The installation is correct."
  echo "The service $service:$svcsname can be started now"
else
  echo "The installation is not correct. run $bindir/setup -r $filename again"
fi
