#!/bin/bash
#
# Copyright 2012 Doerzbach Engineering GmbH 
#
# http://www.doerzbach.com
# 
# Written by Armin Doerzbach 
# 

. `dirname $0`/functions.bash

export LANG=C
usage(){
  log "E" "Usage: `basename $0` <check|runonce|start|lockoutkey|reenablekey> <servicename>"
  exit 255
}
if [ $# -ne 2 ] ; then
   usage
fi
if [ -r $basedir/etc/svcsdefinition/$2.remote ] ; then
  remote=true
  ending=".remote"
fi
if [ -r $basedir/etc/svcsdefinition/$2 ] ; then
  remote=false
  ending=""
fi
. $basedir/etc/svcsdefinition/${2}$ending
if [ -r $basedir/etc/svcsdefinition/$2.remote -a -r $basedir/etc/svcsdefinition/$2.remote ] ; then
  thisuser=`id | sed 's/[^(]*(\([^)]*\).*/\1/g'`
  if [ $thisuser = $dstuser ] ; then
     remote=true  
  else
     remote=false
  fi
fi

if [ -z "$svcsname" ] ; then
  log "A" "no environment is set up for service $2"
  exit 1
fi

generate_arguments(){
  args="-i $interval -k $keepsnaps -r $retries"
  if [ "$remote" = "false" ] ; then
    args="$args -c $dstuser@$dsthost -p $basedir/etc/ssh_keys/$srcuser/$svcsname"
  else
    args="$args -c $srcuser@$srchost -p $basedir/etc/ssh_keys/$dstuser/$svcsname"
  fi
     
  if [ "$beforescript" != "none" ] ; then
     args="$args -b $beforescript"
  fi
  if [ "$afterscript" != "none" ] ; then
     args="$args -a $afterscript"
  fi
  index=0
  while [ $index -lt $number_of_filesytems ] ; do
     let index++
     if [ "$remote" = "false" ] ; then
        args="$args -s ${source_zfs[$index]}"
        args="$args -d ${dest_zfs[$index]}"
     else
        args="$args -d ${source_zfs[$index]}"
        args="$args -s ${dest_zfs[$index]}"
     fi
  done
  echo $args
}
case $1 in
  check) $bindir/zfs_replicate.bash `generate_arguments` -C ;;
  runonce) 
    interval=0 
    if [ $remote = true ] ; then
      export remote
      export dsthost
      echo "You possibly destroy data on the source system $srchost."
      echo "You will be asked to enter the password of the source user $srcuser"
      cat $basedir/etc/ssh_keys/$dstuser/$svcsname.pub| ssh -i /tmp/nokeyfile $srcuser@$srchost "cat >> .ssh/authorized_keys"
    fi
    $bindir/zfs_replicate.bash `generate_arguments` 
    if [ $remote = true ] ; then
      ssh -i $basedir/etc/ssh_keys/$dstuser/$svcsname $srcuser@$srchost "grep -v zfs_replication:$svcsname@$dsthost .ssh/authorized_keys > .ssh/authorized_keys.tmp ;cp .ssh/authorized_keys.tmp .ssh/authorized_keys; rm .ssh/authorized_keys.tmp" 
    fi
  ;;
  start) 
    if [ $remote = false ] ; then
       $bindir/zfs_replicate.bash `generate_arguments` & 
       pid=$!
       sleep 5
       if [ ! -x /proc/$pid ] ; then
         echo "Service dies after 5 seconds already!"
         exit 95
       fi 
    else 
       echo "No service for remote system to start"
    fi
  ;;
  lockoutkey) 
    if [ $remote = true ] ; then
      grep "zfs_replication:$svcsname@$srchost" $HOME/.ssh/authorized_keys > $HOME/.ssh/authorized_keys.lockedout.$svcsname
      grep -v "zfs_replication:$svcsname@$srchost" $HOME/.ssh/authorized_keys > $HOME/.ssh/authorized_keys.tmp
      cp $HOME/.ssh/authorized_keys.tmp $HOME/.ssh/authorized_keys
      rm $HOME/.ssh/authorized_keys.tmp
    else
      echo lockout can only be run on remote system
    fi
  ;;
  reenablekey) 
    if [ $remote = true ] ; then
      cat $HOME/.ssh/authorized_keys.lockedout.$svcsname >> $HOME/.ssh/authorized_keys
      rm $HOME/.ssh/authorized_keys.lockedout.$svcsname
    else
      echo reenablekey can only be run on remote system
    fi
  ;;
  *) usage ;;
esac
